@@ -7,16 +7,17 @@ from ._lru_cache import LRUCache
 
 def cell_len(text: str, _cache: Dict[str, int] = LRUCache(1024 * 4)) -> int:
     """Get the number of cells required to display text.
-    
+
     Args:
         text (str): Text to display.
-    
+
     Returns:
         int: Number of cells required to display the text.
     """
     cached_result = _cache.get(text, None)
     if cached_result is not None:
         return cached_result
+
     _get_size = get_character_cell_size
     total_size = sum(_get_size(character) for character in text)
     if len(text) <= 64:
@@ -24,12 +25,13 @@ def cell_len(text: str, _cache: Dict[str, int] = LRUCache(1024 * 4)) -> int:
     return total_size
 
 
+@lru_cache(maxsize=4096)
 def get_character_cell_size(character: str) -> int:
     """Get the cell size of a character.
-    
+
     Args:
         character (str): A single character.
-    
+
     Returns:
         int: Number of cells (0, 1 or 2) occupied by that character.
     """
@@ -44,10 +46,10 @@ def get_character_cell_size(character: str) -> int:
 @lru_cache(maxsize=4096)
 def _get_codepoint_cell_size(codepoint: int) -> int:
     """Get the cell size of a character.
-    
+
     Args:
         character (str): A single character.
-    
+
     Returns:
         int: Number of cells (0, 1 or 2) occupied by that character.
     """
@@ -78,18 +80,26 @@ def set_cell_size(text: str, total: int) -> str:
     if cell_size < total:
         return text + " " * (total - cell_size)
 
-    _get_character_cell_size = get_character_cell_size
-    character_sizes = [_get_character_cell_size(character) for character in text]
-    excess = cell_size - total
-    pop = character_sizes.pop
-    while excess > 0 and character_sizes:
-        excess -= pop()
-    text = text[: len(character_sizes)]
-    if excess == -1:
-        text += " "
-    return text
+    start = 0
+    end = cell_size
+
+    # Binary search until we find the right size
+    while True:
+        pos = (start + end) // 2
+        before = text[:pos]
+        before_len = cell_len(before)
+        if before_len == total + 1 and cell_len(before[-1]) == 2:
+            return before[:-1] + " "
+        if before_len == total:
+            return before
+        if before_len > total:
+            end = pos
+        else:
+            start = pos
 
 
+# TODO: This is inefficient
+# TODO: This might not work with CWJ type characters
 def chop_cells(text: str, max_size: int, position: int = 0) -> List[str]:
     """Break text in to equal (cell) length strings."""
     _get_character_cell_size = get_character_cell_size
